@@ -1,18 +1,22 @@
 test_that("input validation works", {
-  expect_error(is_within(ip_network(), ip_network()), "not an ip_address vector")
-  expect_error(is_within(ip_address(), ip_address()), "not an ip_network vector")
-  expect_error(is_within(ip_address("1.2.3.4"), ip_network()), "not equal to length")
+  expect_error(is_within(ip_network(), ip_network()))
+  expect_error(is_within(ip_address(), ip_address()))
+  expect_error(is_within(ip_address(rep("1.2.3.4", 3)), ip_network(rep("1.2.3.4/32", 2))))
 
-  expect_error(is_within_any(ip_network(), ip_network()), "not an ip_address vector")
-  expect_error(is_within_any(ip_address(), ip_address()), "not an ip_network vector")
+  expect_error(is_within_any(ip_network(), ip_network()))
+  expect_error(is_within_any(ip_address(), ip_address()))
 
-  expect_error(is_subnet(ip_address(), ip_network()), "not an ip_network vector")
-  expect_error(is_subnet(ip_network(), ip_address()), "not an ip_network vector")
-  expect_error(is_subnet(ip_network("1.2.3.4/32"), ip_network()), "not equal to length")
+  expect_error(overlaps(ip_address(), ip_network()))
+  expect_error(overlaps(ip_network(), ip_address()))
+  expect_error(overlaps(ip_network(rep("1.2.3.4/32", 3)), ip_network(rep("1.2.3.4/32", 2))))
 
-  expect_error(is_supernet(ip_address(), ip_network()), "not an ip_network vector")
-  expect_error(is_supernet(ip_network(), ip_address()), "not an ip_network vector")
-  expect_error(is_supernet(ip_network("1.2.3.4/32"), ip_network()), "not equal to length")
+  expect_error(is_subnet(ip_address(), ip_network()))
+  expect_error(is_subnet(ip_network(), ip_address()))
+  expect_error(is_subnet(ip_network(rep("1.2.3.4/32", 3)), ip_network(rep("1.2.3.4/32", 2))))
+
+  expect_error(is_supernet(ip_address(), ip_network()))
+  expect_error(is_supernet(ip_network(), ip_address()))
+  expect_error(is_supernet(ip_network(rep("1.2.3.4/32", 3)), ip_network(rep("1.2.3.4/32", 2))))
 })
 
 test_that("is_within works", {
@@ -22,8 +26,8 @@ test_that("is_within works", {
     "192.168.3.255",
     "192.168.4.0"
   ))
-  netw <- rep(ip_network("192.168.0.0/22"), length(addr))
-
+  netw <- ip_network("192.168.0.0/22")
+  expect_equal(is_within(addr, rep(netw, 4)), c(FALSE, TRUE, TRUE, FALSE))
   expect_equal(is_within(addr, netw), c(FALSE, TRUE, TRUE, FALSE))
 
   addr <- ip_address(c(
@@ -32,8 +36,8 @@ test_that("is_within works", {
     "2001:db8:fff:ffff:ffff:ffff:ffff:ffff",
     "2001:db9::"
   ))
-  netw <- rep(ip_network("2001:db8::/36"), length(addr))
-
+  netw <- ip_network("2001:db8::/36")
+  expect_equal(is_within(addr, rep(netw, 4)), c(FALSE, TRUE, TRUE, FALSE))
   expect_equal(is_within(addr, netw), c(FALSE, TRUE, TRUE, FALSE))
 })
 
@@ -54,7 +58,7 @@ test_that("is_within_any works", {
   )
 })
 
-test_that("is_subnet and is_supernet work", {
+test_that("network-in-network works", {
   net1 <- ip_network(c(
     "192.167.255.255/32",
     "192.168.0.0/32",
@@ -62,8 +66,10 @@ test_that("is_subnet and is_supernet work", {
     "192.168.4.0/32",
     "192.168.0.0/20"
   ))
-  net2 <- rep(ip_network("192.168.0.0/22"), length(net1))
+  net2 <- ip_network("192.168.0.0/22")
 
+  expect_equal(overlaps(net1, net2), c(FALSE, TRUE, TRUE, FALSE, TRUE))
+  expect_equal(overlaps(net2, net1), c(FALSE, TRUE, TRUE, FALSE, TRUE))
   expect_equal(is_subnet(net1, net2), c(FALSE, TRUE, TRUE, FALSE, FALSE))
   expect_equal(is_subnet(net2, net1), c(FALSE, FALSE, FALSE, FALSE, TRUE))
   expect_equal(is_supernet(net1, net2), c(FALSE, FALSE, FALSE, FALSE, TRUE))
@@ -76,8 +82,10 @@ test_that("is_subnet and is_supernet work", {
     "2001:db9::/128",
     "2001:db8::/30"
   ))
-  net2 <- rep(ip_network("2001:db8::/36"), length(net1))
+  net2 <- ip_network("2001:db8::/36")
 
+  expect_equal(overlaps(net1, net2), c(FALSE, TRUE, TRUE, FALSE, TRUE))
+  expect_equal(overlaps(net2, net1), c(FALSE, TRUE, TRUE, FALSE, TRUE))
   expect_equal(is_subnet(net1, net2), c(FALSE, TRUE, TRUE, FALSE, FALSE))
   expect_equal(is_subnet(net2, net1), c(FALSE, FALSE, FALSE, FALSE, TRUE))
   expect_equal(is_supernet(net1, net2), c(FALSE, FALSE, FALSE, FALSE, TRUE))
@@ -92,6 +100,9 @@ test_that("multi-space comparisons fail", {
   expect_false(is_within_any(ip_address("192.168.0.1"), ip_network(c("2001:db8::/36", "192.168.0.0/32"))))
   expect_false(is_within_any(ip_address("192.168.0.1"), ip_network(c("2001:db8::/36", "2001:db8::/64"))))
 
+  expect_false(overlaps(ip_network("2001:db8::/36"), ip_network("192.168.0.0/22")))
+  expect_false(overlaps(ip_network("192.168.0.0/22"), ip_network("2001:db8::/36")))
+
   expect_false(is_subnet(ip_network("2001:db8::/36"), ip_network("192.168.0.0/22")))
   expect_false(is_subnet(ip_network("192.168.0.0/22"), ip_network("2001:db8::/36")))
 
@@ -107,6 +118,9 @@ test_that("missing values work", {
   expect_true(is_within_any(ip_address("192.168.0.1"), ip_network(c("192.168.0.0/24", NA))))
   expect_false(is_within_any(ip_address("192.168.0.1"), ip_network(c("192.168.0.0/32", NA))))
 
+  expect_equal(overlaps(ip_network(NA), ip_network("192.168.0.0/24")), NA)
+  expect_equal(overlaps(ip_network("192.168.0.0/24"), ip_network(NA)), NA)
+
   expect_equal(is_subnet(ip_network(NA), ip_network("192.168.0.0/24")), NA)
   expect_equal(is_subnet(ip_network("192.168.0.0/24"), ip_network(NA)), NA)
 
